#define PY_SSIZE_T_CLEAN 1
#include <Python.h>

#include <sys/types.h>
#include <sys/socket.h>

#if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
/* This may cause some warnings, but if you want to get rid of them, upgrade
 * your Python version.  */
typedef int Py_ssize_t;
#endif

static PyObject *listreceiver_listreceiver(PyObject *self, PyObject *args, PyObject *keywds);

static PyMethodDef listreceiver_methods[] = {
	{"listreceiver", (PyCFunction) listreceiver_listreceiver, METH_VARARGS | METH_KEYWORDS,
	 NULL},
	{NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC initlistreceiver(void) {
	PyObject *module;
	module = Py_InitModule("listreceiver", listreceiver_methods);

	if (!module) {
		return;
	}

	if (-1 == PyModule_AddIntConstant(module, "SCM_RIGHTS", SCM_RIGHTS)) {
		return;
	}
}


static PyObject *listreceiver_listreceiver(PyObject *self, PyObject *args, PyObject *keywds) {
	int fd,i,flags = 0;
	struct msghdr message_header = {0};
	struct iovec iov[1];
	PyObject *ancillary = NULL;
	static char *kwlist[] = {"fd", "data", "flags", "ancillary", NULL};

	if (!PyArg_ParseTupleAndKeywords(
			args, keywds, "it#|iO:listrec", kwlist,
			&fd,
			&iov[0].iov_base,
			&iov[0].iov_len,
			&flags,
			&ancillary)) {
		return NULL;
	}

	if (!PyList_Check(ancillary)) {
		PyErr_Format(PyExc_TypeError,
			"argument 3 expected list, got %s",
			ancillary->ob_type->tp_name);
		return NULL;
	}

	PyObject *iterator = PyObject_GetIter(ancillary);
	PyObject *item = NULL;

	if (iterator == NULL) {
		return NULL;
	}

	/* First we need to know how big the buffer needs to be in order to
	   have enough space for all of the messages. */
	while ( (item = PyIter_Next(iterator)) ) {
		int type, level;
		Py_ssize_t data_len;
		char *data;
		if (!PyArg_ParseTuple(
				item, "iit#:ancillary data (level, type, data)",
				&level, &type, &data, &data_len)) {
			Py_DECREF(item);
			Py_DECREF(iterator);
			return NULL;
	    }
		printf("First call\n");
		printf("level %x\n",level);
		printf("type  %x\n",type);
		for (i=0;i<data_len;i++) {
			printf("%02x ",data[i]);
		}
		printf("(%d)\n",(int)data_len);

		Py_DECREF(item);

	}
	Py_DECREF(iterator);
	iterator = NULL;

	iterator = PyObject_GetIter(ancillary); /* again */
	item = NULL;

	if (!iterator) {
		return NULL;
	}

	while ( (item = PyIter_Next(iterator)) ) {
		int data_len, type, level;
		unsigned char *data;

		if (!PyArg_ParseTuple(item,
					"iit#:listrec ancillary data (level, type, data)",
					&level,
					&type,
					&data,
					&data_len)) {
			Py_DECREF(item);
			Py_DECREF(iterator);
			free(message_header.msg_control);
			return NULL;
		}
		printf("Second call\n");
		printf("level %x\n",level);
		printf("type  %x\n",type);
		for (i=0;i<data_len;i++) {
			printf("%02x ",data[i]);
		}
		printf("(%d)\n",data_len);
		Py_DECREF(item);
	}

	Py_DECREF(iterator);

	if (PyErr_Occurred()) {
		return NULL;
	}
	return Py_BuildValue("n", 1);
}
